@@ -126,6 +126,8 @@ typedef struct GokKeyboard {
 	gboolean bDynamicallyCreated;
 	gint NumberRows;
 	gint NumberColumns;
+        gint keyWidth;
+        gint keyHeight;
 	gboolean bRequiresLayout;
 	gboolean bLaidOut;
 	gboolean bSupportsWordCompletion;
