@@ -2061,8 +2061,8 @@ gok_spy_is_primary_container (Accessible *accessible)
 	return is_application;
     }
     return (role == SPI_ROLE_HTML_CONTAINER || role == SPI_ROLE_SCROLL_PANE || 
-	role == SPI_ROLE_FRAME || role == SPI_ROLE_ROOT_PANE || 
-        role == SPI_ROLE_APPLICATION);
+	role == SPI_ROLE_FRAME || role == SPI_ROLE_DIALOG ||
+	role == SPI_ROLE_ROOT_PANE || role == SPI_ROLE_APPLICATION);
 }
 
 /** 
@@ -2570,13 +2570,22 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 			else if (keyboard_ui_flags.data.toolbars && 
 				 parent_role == SPI_ROLE_TOOL_BAR)
 			{
+                                GokSpyUIFlags toolbar_search_flags;
+                                toolbar_search_flags.value = 0;
 				if (keyboard_ui_flags.data.gui) 
 					flags.data.is_ui = TRUE; /* toolbar items are ui elements too */
 				ui_flags.data.toolbars = flags.data.is_toolbar_item = TRUE;
-				if (!gok_spy_find_and_append_toolbar_items (child, flags)) 
+				if (gok_spy_find_and_append_toolbar_items (child, flags)) 
 				{
-				    ui_flags.value |= gok_spy_search_component_list (child, keyboard_ui_flags, context_flags, search_depth).value;
-				}
+                                    toolbar_search_flags.data.menus = keyboard_ui_flags.data.menus;
+                                }
+                                else
+                                {
+                                    toolbar_search_flags.value = keyboard_ui_flags.value;
+                                }
+                                ui_flags.value |= gok_spy_search_component_list (child,
+                                                                                 toolbar_search_flags,
+                                                                                 context_flags, search_depth).value;
 			}
 			if ((keyboard_ui_flags.data.gui || keyboard_ui_flags.data.editable_text) && 
 			    !flags.data.is_menu && !flags.data.is_toolbar_item && gok_spy_is_ui (child, parent, role))
@@ -2594,7 +2603,6 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 			{
 			        AccessibleNodeFlags non_context_flags = flags;
 				non_context_flags.data.inside_html_container = FALSE;
-
 				if (non_context_flags.value) 
 				{
 				    _priv_ui_nodes = gok_spy_append_node (_priv_ui_nodes, child, flags);
