@@ -36,6 +36,7 @@ use wnck_window_is_active to figure out which window in list is active (since no
 #include "gok-log.h"
 #include "main.h"
 #include "gok-windowlister.h"
+#include "gdk/gdkx.h"
 
 /* private variables */
 static GList* m_window_list = NULL;
@@ -43,6 +44,123 @@ static GList* m_window_list = NULL;
 /* private functions */
 void gok_windowlister_refreshList (void);
 gboolean gok_windowlister_isValid ( void* pWindow );
+void _modified_wnck_window_activate    (WnckWindow *window);
+void _modified_wnck_workspace_activate (WnckWorkspace *space,
+					WnckWindow    *window);
+
+/* private cut-n-paste functions from libwnck/xutils.c */
+void _wnck_activate (Screen *screen,
+		     Window  xwindow,
+		     Time    timestamp);
+
+void _wnck_activate_workspace (Screen *screen,
+			       int     new_active_space,
+			       Time    timestamp);
+
+
+void
+_wnck_activate (Screen *screen,
+                Window  xwindow,
+                Time    timestamp)
+{
+  XEvent xev;
+  Atom   net_active_window_atom;
+
+  /* Deviating from libwnck here because copying _wnck_atom_get sounds like
+   * overkill.  This is slightly slower, but oh well.
+   */
+  net_active_window_atom = XInternAtom (gdk_display,
+					"_NET_ACTIVE_WINDOW",
+					FALSE);
+  
+  xev.xclient.type = ClientMessage;
+  xev.xclient.serial = 0;
+  xev.xclient.send_event = True;
+  xev.xclient.display = gdk_display;
+  xev.xclient.window = xwindow;
+  xev.xclient.message_type = net_active_window_atom;
+  xev.xclient.format = 32;
+  xev.xclient.data.l[0] = 2;
+  xev.xclient.data.l[1] = timestamp;
+  xev.xclient.data.l[2] = 0;
+  xev.xclient.data.l[3] = 0;
+  xev.xclient.data.l[4] = 0;
+
+  XSendEvent (gdk_display,
+	      RootWindowOfScreen (screen),
+              False,
+	      SubstructureRedirectMask | SubstructureNotifyMask,
+	      &xev); 
+}
+
+void
+_wnck_activate_workspace (Screen *screen,
+                          int     new_active_space,
+                          Time    timestamp)
+{
+  XEvent xev;
+  Atom   net_current_desktop_atom;
+  
+    /* Deviating from libwnck here because copying _wnck_atom_get sounds like
+   * overkill.  This is slightly slower, but oh well.
+   */
+  net_current_desktop_atom = XInternAtom (gdk_display,
+					  "_NET_CURRENT_DESKTOP",
+					  FALSE);
+
+  xev.xclient.type = ClientMessage;
+  xev.xclient.serial = 0;
+  xev.xclient.send_event = True;
+  xev.xclient.display = gdk_display;
+  xev.xclient.window = RootWindowOfScreen (screen);
+  xev.xclient.message_type = net_current_desktop_atom;
+  xev.xclient.format = 32;
+  xev.xclient.data.l[0] = new_active_space;
+  xev.xclient.data.l[1] = timestamp;
+  xev.xclient.data.l[2] = 0;
+  xev.xclient.data.l[3] = 0;
+  xev.xclient.data.l[4] = 0;
+
+  XSendEvent (gdk_display,
+	      RootWindowOfScreen (screen),
+              False,
+	      SubstructureRedirectMask | SubstructureNotifyMask,
+	      &xev);
+}
+
+void
+_modified_wnck_window_activate (WnckWindow *window)
+{
+  Screen  *screen;
+  Window   xid;
+  XWindowAttributes attrs;
+
+  xid = wnck_window_get_xid (window);
+  XGetWindowAttributes(gdk_display, xid, &attrs);
+
+  _wnck_activate (attrs.screen,
+		  xid,
+		  0);
+}
+
+/* The WnckWindow is merely used to get the x screen */
+void
+_modified_wnck_workspace_activate (WnckWorkspace *space,
+				   WnckWindow    *window)
+{
+  Screen  *screen;
+  Window   xid;
+  XWindowAttributes attrs;
+
+  xid = wnck_window_get_xid (window);
+  XGetWindowAttributes(gdk_display, xid, &attrs);
+
+  _wnck_activate_workspace (attrs.screen,
+			    wnck_workspace_get_number (space),
+			    0);
+}
+
+
 
 /**
 * gok_windowlist_show
@@ -196,10 +314,10 @@ void gok_windowlister_onKey (GokKey* pKey)
 		    workspace = wnck_window_get_workspace (window_list_entry->data);
 		    if (workspace != NULL) 
 		    {
-		        wnck_workspace_activate (workspace);
+		        _modified_wnck_workspace_activate (workspace,window_list_entry->data);
 		    }
 			bSuccess = TRUE;
-			wnck_window_activate (window_list_entry->data);
+			_modified_wnck_window_activate (window_list_entry->data);
 			break;
 		}
         window_list_entry = window_list_entry->next;
@@ -212,13 +330,14 @@ void gok_windowlister_onKey (GokKey* pKey)
 	/* check to see if window list entry is still valid */
 	if (gok_windowlister_isValid(pKey->pGeneral) == TRUE)
 	{
-	    workspace = wnck_window_get_workspace (pKey->pGeneral);
+		workspace = wnck_window_get_workspace (pKey->pGeneral);
 	    if (workspace != NULL) 
 	    {
-	        wnck_workspace_activate (workspace);
+	        _modified_wnck_workspace_activate (workspace,
+						   pKey->pGeneral);
 	    }
 		bSuccess = TRUE;
-		wnck_window_activate (pKey->pGeneral);
+		_modified_wnck_window_activate (pKey->pGeneral);
 	}
 	
 #endif
@@ -234,6 +353,7 @@ void gok_windowlister_onKey (GokKey* pKey)
 void gok_windowlister_refreshList ()
 {
     WnckScreen *screen;
+    static gboolean client_type_set = FALSE;
     
     /* do not free the list here! this seems to cause segv -- need to look at library code 
 	if (m_window_list != NULL)
@@ -241,7 +361,19 @@ void gok_windowlister_refreshList ()
 		g_list_free(m_window_list);
 	}
 	*/
-	
+
+    /* Register as a pager, so that the window manager treats us the
+     * same as a taskbar (i.e. having global awareness of apps) rather
+     * than as a normal app (i.e. primarily only knowing about and
+     * having UI for dealing with my own windows).  Doesn't matter for
+     * most cases, but some WMs in some cases treat the two
+     * differently.
+     */
+    if (!client_type_set) {
+	wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);
+	client_type_set = TRUE;
+    }
+
     screen = wnck_screen_get_default ();
     wnck_screen_force_update (screen);
 	
