@@ -88,6 +88,10 @@
 
 static gboolean respawn_on_segv = TRUE;
 
+static gboolean queue_input_events = TRUE;
+
+static guint input_idle_handler = 0;
+
 static GokApplication *_gok_app = NULL;
 
 /* pointer to the first keyboard in the list of keyboards */
@@ -141,6 +145,11 @@ typedef struct _GokArgs {
 
 static GokArgs gok_args;
 
+static struct {
+    gint n_axes;
+    int *axes;
+} gok_motion_event = {0, 0x0000};
+
 /* Private functions */
 static void gok_main_read_rc (void);
 static void gok_main_initialize_access_methods (GokArgs *args);
@@ -1096,6 +1105,40 @@ gok_main_object_state_listener (Accessible* pAccessible)
 	}
 }	
 
+/** 
+ * gok_main_check_input_queue (gpointer data):
+ *
+ * Check the extended input event queue and act on the most recent motion event,
+ * and discard other motion events.
+ * 
+ * returns: FALSE as specified in g_idle_add ().
+ **/
+static gboolean gok_main_idle_input (gpointer data)
+{
+    input_idle_handler = 0;
+    if ((gok_motion_event.n_axes > 0) && (gok_motion_event.n_axes < 4)) {
+	gok_scanner_input_motion (gok_motion_event.axes, gok_motion_event.n_axes);
+    }
+    return FALSE;
+}
+
+void
+gok_main_queue_input_event (int *motion_data, gint n_axes)
+{
+    if (n_axes > 0 && n_axes < 4) {
+	if (gok_motion_event.axes)
+	    g_free (gok_motion_event.axes);
+	gok_motion_event.axes = g_new (int, n_axes);
+	gok_motion_event.n_axes = n_axes;
+	memcpy (gok_motion_event.axes, motion_data, n_axes * sizeof (int));
+	if (input_idle_handler == 0) {
+	    input_idle_handler = g_idle_add (gok_main_idle_input, NULL);
+	}
+    }
+    else
+	g_warning ("Motion Event has an invalid number of axes!");
+}
+
 
 /**
 * gok_main_motion_listener
@@ -1105,6 +1148,9 @@ gok_main_object_state_listener (Accessible* pAccessible)
 * @timestamp: long int, ignored
 *
 * This handler is called each time there is a motion event from the connected input device.
+* We queue the input event for processing during idle, as the rate of input events
+* may exceed our ability to process them, otherwise, and we only need to consider the most
+* recent motion event.
 */
 void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long timestamp)
 {
@@ -1118,7 +1164,10 @@ void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long ti
 			mods, timestamp);
 		gok_log_leave ();
 #endif
-		gok_scanner_input_motion (motion_data, n_axes);
+		if (queue_input_events)
+		    gok_main_queue_input_event (motion_data, n_axes);
+		else
+		    gok_scanner_input_motion (motion_data, n_axes); 
 
 		if (gok_data_get_drive_corepointer ()) {
 			Display *display;
@@ -2499,6 +2548,9 @@ gok_main_close()
 	GokKeyboard* pKeyboard;
 	GokKeyboard* pKeyboardTemp;
 
+	/* get rid of input idle handler */
+	if (input_idle_handler) g_source_remove (input_idle_handler);
+
 	/* restore the corepointer, if it was changed... */
 	gok_input_restore_corepointer ();
 
@@ -2894,7 +2946,9 @@ gok_main_help_cb (gpointer data)
 	GError *error = NULL;
 	gchar *helpuri = data;
 	/* TODO: detect error launching help, and give informative message */
-	gnome_help_display_desktop (NULL, "gok", helpuri, NULL, &error);
+	/* TODO: remove hack for link below, if and when this method is called from more than one dialog */
+
+	gnome_help_display_desktop (NULL, "gnome-access-guide", helpuri, "sysadmin-37", &error);
 }
 
 void
@@ -2947,6 +3001,7 @@ gok_main_warn_corepointer (gboolean always, gboolean configuring, gboolean opt_o
 gboolean
 gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback okfunc, gboolean opt_out)
 {
+
 	if (always || gok_scanner_current_state_uses_corepointer ()) 
 	{
 		GtkWidget*	button;
@@ -2968,7 +3023,7 @@ gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback
 			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
 					   button);
 			g_signal_connect_swapped (G_OBJECT (button), "clicked", 
-						  G_CALLBACK (gok_main_help_cb), "gok.xml");
+						  G_CALLBACK (gok_main_help_cb), "sysadmin.xml");
 			
 			button = gtk_button_new_from_stock(GTK_STOCK_PREFERENCES);
 			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
